@@ -172,7 +172,8 @@ def form_info(state):
 
     # Generate a paging URL for the form
     document = state[state.manage_config.var_name]
-    params = {state.manage_config.var_name: document._id}
+    params = {state.manage_config.var_name: [document._id]}
+    form_info['paging_params'] = params
     form_info['paging_query_string'] = urlencode(params, True)
 
     # Predefined form information overrides information defined by the link
