@@ -17,9 +17,10 @@ from manhattan.manage.views import utils
 
 # Optional imports
 try:
-    from manhattan.assets import Asset
-    from manhattan.assets.fields import AssetField
+    from manhattan.assets import Asset, ImageSet
+    from manhattan.assets.fields import AssetField, ImageSetField
     from manhattan.assets.transforms.base import BaseTransform
+    from manhattan.assets.transforms.images import Fit, Output
     __assets_supported__ = True
 
 except ImportError as e:
@@ -40,15 +41,15 @@ __all__ = [
 
 
 def authenticate(
-        user_g_key='user',
-        sign_in_endpoint='manage_users.sign_in',
-        sign_out_endpoint='manage_users.sign_out',
-        mfa_required=None,
-        mfa_authorize_endpoint='manage_users.mfa_auth',
-        mfa_authorized_attr='mfa_authorized',
-        mfa_enable_endpoint='manage_users.mfa_enable',
-        mfa_enabled_attr='mfa_enabled',
-    ):
+    user_g_key='user',
+    sign_in_endpoint='manage_users.sign_in',
+    sign_out_endpoint='manage_users.sign_out',
+    mfa_required=None,
+    mfa_authorize_endpoint='manage_users.mfa_auth',
+    mfa_authorized_attr='mfa_authorized',
+    mfa_enable_endpoint='manage_users.mfa_enable',
+    mfa_enabled_attr='mfa_enabled'
+):
     """
     Authenticate the caller has permission to call the view.
 
@@ -399,13 +400,17 @@ def store_assets():
         # Identify asset fields within the form
         document = state[state.manage_config.var_name]
         asset_fields = []
+        image_set_fields = []
 
         for field in document.get_fields():
 
-            if not isinstance(getattr(state.form, field, None), AssetField):
-                continue
+            form_field = getattr(state.form, field, None)
 
-            asset_fields.append(field)
+            if isinstance(form_field, AssetField):
+                asset_fields.append(field)
+
+            if isinstance(form_field, ImageSetField):
+                image_set_fields.append(field)
 
         # Build a list of assets to make permanent (persist), to generate new
         # variations for (transform) and to remove.
@@ -413,6 +418,7 @@ def store_assets():
         assets_to_transform = []
         assets_to_remove = []
 
+        # Asset fields
         for field in asset_fields:
 
             asset = state.form_data.get(field)
@@ -441,18 +447,36 @@ def store_assets():
             # Build the transform instructions required to regenerate
             # variations for the asset.
 
-            # Get variations defined for the gallery field against the manage
-            # config.
+            # Get variations defined for the field against the manage config.
             variations = state.manage_config.asset_variations.get(field, {})
 
+            # Copy the variations structure
+            variations = {k: [i for i in v] for k, v in variations.items()}
+
+            if '--thumb--' in asset.variations:
+
+                # Cater for regenerating the thumbnail.
+                variations['--thumb--'] = flask.current_app.config.get(
+                    'ASSET_THUMB_VARIATION',
+                    [
+                        Fit(480),
+                        Output('jpg', 75)
+                    ]
+                )
+
             if asset.variations:
 
                 # Check for local transforms against the asset which are added
-                # ro and may override variations defined for the gallery
-                # field.
+                # to and may override variations defined for the field.
                 for variation_name, variation_asset \
                         in asset.variations.items():
 
+                    if variation_name in variations:
+
+                        # Don't use the local transforms if there's
+                        # transforms defined the variation.
+                        continue
+
                     if not isinstance(variation_asset, Asset):
                         variation_asset = Asset(variation_asset)
 
@@ -477,6 +501,165 @@ def store_assets():
                     ]
                 ))
 
+        # Image set fields
+        for field in image_set_fields:
+
+            image_set = state.form_data.get(field)
+
+            if image_set:
+                current_image_set = document.get(field)
+
+                images_to_regenerate = {}
+
+                for version, asset in image_set.images.items():
+
+                    if asset.temporary:
+
+                        # This a new asset so persist it
+                        assets_to_persist.append(asset)
+
+                        if current_image_set \
+                                and current_image_set.images.get(version):
+
+                            # There's an old version of this asset so remove
+                            # it.
+                            assets_to_remove.append(
+                                current_image_set.images[version]
+                            )
+
+                        # Determine if we need to generate variations of this
+                        # asset.
+                        if version == image_set.base_version:
+                            for v in asset.variations.keys():
+                                if v == image_set.base_version \
+                                        and image_set.base_transforms.get(v):
+
+                                    images_to_regenerate[version] = asset
+                                    break
+
+                                if v not in image_set.images \
+                                        and image_set.base_transforms.get(v):
+
+                                    images_to_regenerate[version] = asset
+                                    break
+
+                        else:
+                            if image_set.base_transforms.get(version):
+                                images_to_regenerate[version] = asset
+
+                    else:
+
+                        # Determine if we need to generate variations of this
+                        # asset.
+                        modified = state.form[field].modified_base_transforms
+
+                        if version == image_set.base_version:
+                            for v in asset.variations.keys():
+
+                                if v == image_set.base_version \
+                                        and v in modified:
+                                    images_to_regenerate[version] = asset
+                                    break
+
+                                if v not in image_set.images and v in modified:
+                                    images_to_regenerate[version] = asset
+                                    break
+
+                        else:
+                            if version in modified:
+                                images_to_regenerate[version] = asset
+
+                if current_image_set:
+                    for version, asset in current_image_set.images.items():
+                        if not image_set.images.get(version):
+
+                            # The new image set doesn't contain this version
+                            # so we need to remove the associated image asset.
+                            assets_to_remove.append(asset)
+
+                # Build the transform instructions required to regenerate
+                # variations for the assets in the image set.
+
+                for version, asset in images_to_regenerate.items():
+
+                    # Get variations defined for the field against the manage
+                    # config.
+                    variations = state.manage_config.asset_variations.get(
+                        field,
+                        {}
+                    )
+
+                    # Copy the variations structure
+                    variations = {
+                        k: [i for i in v]
+                        for k, v in variations.items()
+                    }
+
+                    # If this isn't the base version of the image set remove
+                    # any variations not related to the verison.
+                    if version != image_set.base_version:
+                        if version in variations:
+                            variations = {version: variations[version]}
+                        else:
+                            variations = {}
+
+                    if '--thumb--' in asset.variations:
+
+                        # Cater for regenerating the thumbnail.
+                        variations['--thumb--'] = flask.current_app.config.get(
+                            'ASSET_THUMB_VARIATION',
+                            [
+                                Fit(480),
+                                Output('jpg', 75)
+                            ]
+                        )
+
+                        if version in image_set.base_transforms:
+                            variations['--thumb--'] = [
+                                BaseTransform.from_json_type(t)
+                                for t in image_set.base_transforms[version]
+                            ] + variations['--thumb--']
+
+                    # Ensure the system set '--draft--' variation is never
+                    # updated.
+                    variations.pop('--draft--', None)
+
+                    if variations:
+
+                        # Build the local transforms for the asset using the
+                        # base transforms and defined transforms. Unlike with
+                        # standard assets, assets in image sets don't have
+                        # separate base transforms (these are stored directly
+                        # against the image set instead).
+
+                        for k, v in variations.items():
+
+                            base_transforms = []
+
+                            if k in image_set.base_transforms:
+                                if version != image_set.base_version \
+                                        or k not in image_set.images \
+                                        or k == image_set.base_version:
+
+                                    base_transforms = [
+                                        BaseTransform.from_json_type(t)
+                                        for t in image_set.base_transforms[k]
+                                    ]
+
+                            variations[k] = base_transforms + variations[k]
+
+                        # Add the tranform information for the asset (the asset,
+                        # variations and base transforms).
+                        assets_to_transform.append((asset, variations, []))
+
+            else:
+                if document.get(field):
+
+                    # Image set has been removed so remove all image assets
+                    # associated with it.
+                    for image in document[field].images:
+                        assets_to_remove.append(image)
+
         # Store assets permanently
         asset_mgr = flask.current_app.asset_mgr
         asset_mgr.persist_many(assets_to_persist)
@@ -488,7 +671,9 @@ def store_assets():
             {a[0].key: a[2] for a in assets_to_transform}
         )
 
-        if asset_fields:
+        if asset_fields or image_set_fields:
+
+            update_fields = asset_fields + image_set_fields
 
             # Update the document
             if hasattr(document, 'logged_update'):
@@ -496,19 +681,20 @@ def store_assets():
                     state.manage_user,
                     {
                         field: state.form_data.get(field) or None
-                        for field in asset_fields
-                    }
+                        for field in update_fields
+                    },
+                    *update_fields
                 )
 
             else:
-                for field in asset_fields:
+                for field in update_fields:
                     setattr(
                         document,
                         field,
                         state.form_data.get(field) or None
                     )
 
-                document.update(*asset_fields)
+                document.update(*update_fields)
 
         if state.remove_assets:
 
